#include "m33.h"


V3& M33::operator[](int i) {

	return rows[i];

}

/*
m00 m01 m02   v0     (m00*v0 + m01*v1 + m02*v2)
m10 m11 m12   v1  =
m20 m21 m22   v2
*/

V3 M33::operator*(V3 v) {

	V3 ret;
	M33 &m = *this;
	ret[0] = m[0] * v;
	ret[1] = m[1] * v;
	ret[2] = m[2] * v;
	return ret;
}

ostream& operator<<(ostream& ostr, M33 m) {

	return ostr << m[0] << m[1] << m[2];

}


V3 M33::GetColumn(int i) {

	return V3(rows[0][i], rows[1][i], rows[2][i]);

}


M33 M33::Inverted() {

	M33 ret;
	V3 a = GetColumn(0), b = GetColumn(1), c = GetColumn(2);
	V3 _a = b ^ c; _a = _a * (1.0f / (a * _a));
	V3 _b = c ^ a; _b = _b * (1.0f / (b * _b));
	V3 _c = a ^ b; _c = _c * (1.0f / (c * _c));
	ret[0] = _a;
	ret[1] = _b;
	ret[2] = _c;

	return ret;

}

void M33::SetColumn(int i, V3 column) {

	M33 &m = *this;
	m[0][i] = column[0];
	m[1][i] = column[1];
	m[2][i] = column[2];

}


void M33::SetRotY(float theta) {

	float thetaRad = 3.1415926f / 180.0f*theta;
	M33 &m = *this;
	m[0] = V3(cosf(thetaRad), 0.0f, sinf(thetaRad));
	m[1] = V3(0.0f, 1.0f, 0.0f);
	m[2] = V3(-sinf(thetaRad), 0.0f, cosf(thetaRad));

}